#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include "portconfig.h"
#include "button.h"
#include "timer.h"


/***************************************************************************
 * 
 * Initialisieren
 * 
 ***************************************************************************/
void buttonInitialize(void)
{
	//Pin als Input setzen
	BUTTON_DDR &= ~(1<<BUTTON_PIN);

	//Pullup altivieren
	BUTTON_PORT |= (1<<BUTTON_PIN);

	buttonstatus = 0;
	buttonlaststatus = 0;
}

/***************************************************************************
 * 
 * Tasterzustand ermitteln
 * 
 ***************************************************************************/
void buttonReadStatus(void)
{
	//Status sichern
	buttonlaststatus = buttonstatus;

	buttonstatus = 0;

	//BUTTON_PIN ist Low-Aktiv!!!!

	//War Button gedrckt?
	if (bit_is_set(buttonlaststatus, BTN_PRESSING))
	{
		//Wird er noch gedrckt?
		if (bit_is_clear(BUTTON_PINR, BUTTON_PIN))
		{
			buttonstatus |= (1<<BTN_PRESSING);

			//Wie lange schon wird er gedrckt?
			if (bit_is_clear(buttonlaststatus, BTN_LONG))
			{
				buttonpressduration = ((TIMER0_MAX_INT+timer0IntervalCount-buttonstartpress)%TIMER0_MAX_INT);
					
				if (buttonpressduration > BTN_LONG_WAIT_CMP)
				{
					//Als Lang-Gedrckt markieren
					buttonstatus |= (1<<BTN_LONG);
				}
			}
			else
				//War vorher schon als Lang markiert, also bernehmen
				buttonstatus |= (1<<BTN_LONG);
		}
		else
		{
			//Losgelassen
			buttonstatus |= (1<<BTN_PRESSED);
			if (bit_is_set(buttonlaststatus, BTN_LONG))
				buttonstatus |= (1<<BTN_LONG);
		}
	}
	else	//Button war nicht gedrckt
	{
		//Wird er jetzt gedrckt?
		if (bit_is_clear(BUTTON_PINR, BUTTON_PIN))
		{
			//Tastendruck - Start
			buttonstatus |= (1<<BTN_PRESSING);
			buttonstartpress = timer0IntervalCount;
		}
	}
}


