/***************************************************************************
 *  Statusled ist jeweils eine 2-PIN-Duo-LED
 * 	4 - Zustnde:
 *		OFF 	Beide Pins aus
 *		GREEN	PinA = ON, PinB = OFF
 *		RED		PinA = OFF, PinB = ON
 *		YELLOW	Schneller Wechsel von RED/GREEN
 ***************************************************************************/
#include <inttypes.h>
#include <avr/io.h>

#include "portconfig.h";
#include "timer.h";
#include "led.h";

/***************************************************************************
 * 
 * Initialisierung
 * Ausgnge setzen, LEDs aus
 * 
 ***************************************************************************/
void ledInitialize(void)
{
	//Pins als Output setzen
	LED_DDR = ((1<<LED_STAT_PIN)|(1<<LED_PWR_PIN));

	//Beim Start: Alles Aus
	ledstatus = 0;
}

/***************************************************************************
 * 
 * Status fr eine LED setzen
 * "led_pin"	PortPin der LED
 * "status"		ledstatus
 * 
 ***************************************************************************/
void ledSetStatus(uint8_t led_pin, uint8_t status)
{
	//Erstmal LED aus
	LED_PORT &= ~(1 << led_pin);

	//LED aktiv?
	if (status != 0)
	{
		//Wenn LED blinkt, sind wir dann in dem Interval an dem die LED leuchtet?
		//Bzw leuchtet die LED dauerhaft?
		if (((bit_is_set(status, LED_STAT_BLINK))&&(timer0IntervalCount<TIMER0_BLINK_CMP))
		   |(bit_is_clear(status, LED_STAT_BLINK)))
		{
			// LED leuchtet hell
			if (bit_is_set(status, LED_STAT_BRIGHT)) 
				LED_PORT |= (1 << led_pin);
			//LED leuchtet dunkel
			else if (bit_is_clear(status, LED_STAT_BRIGHT))
			{
				//jedes 3. TimerIntervall => LED aus (PWM 33% mit 1kHz)
				if ((timer0IntervalCount & 2)==0)
					LED_PORT |= (1 << led_pin);
				else 
					LED_PORT &= ~(1 << led_pin);
			}

		}
	}
}

/***************************************************************************
 * 
 * LED-Anzeige aktualisieren
 * 
 ***************************************************************************/
void ledRefresh(void)
{
	//Erste LED
	ledSetStatus(LED_STAT_PIN, ledstatus);

	//Zweite LED (Status befindet sich in den 4 hheren Bit
	ledSetStatus(LED_PWR_PIN, (ledstatus>>4));
}


