#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/eeprom.h>

#include "portconfig.h";
#include "led.h";
#include "fet.h";
#include "timer.h"
#include "button.h"
#include "akku.h"
#include "globals.h"
#include "eeprom.h"
#include "adc.h"
#include "selftest.h"

void init(void);

volatile uint8_t doactionflag;

/***************************************************************************
 * 
 * Counter0 Interrupt
 * 
 * f=1Khz
 *
 ***************************************************************************/
SIGNAL(SIG_OVERFLOW0)
{
	doactionflag = 1;
}

/*
*/
void doAction(void)
{
	//Zustzlicher Taktdivider, u.a. fr Blinken der LEDs
	if (++timer0IntervalCount == TIMER0_MAX_INT) 
		timer0IntervalCount = 0;
		
	//Tastenstatus bestimmen
	buttonReadStatus();

	//Und auswerten
	if (bit_is_set(buttonstatus, BTN_PRESSED))
	{
		if (bit_is_set(buttonstatus, BTN_LONG))
		{
			//Langer Tastendruck
			switch (controllerstatus)
			{
				case STAT_OFF: 		controllerstatus = STAT_ABBLEND; break;
				case STAT_ABBLEND:  controllerstatus = STAT_OFF; break;	
				case STAT_FERN:		controllerstatus = STAT_OFF; break;
					
			}
		}
		else
		{
			//Kurzer Tastendruck
			switch (controllerstatus)
			{
				case STAT_OFF: 	break;
				case STAT_ABBLEND:  controllerstatus = STAT_FERN; break;	
				case STAT_FERN:		controllerstatus = STAT_ABBLEND; break;			
			}	
		}
	}

	//LED-Anzeige aktualisieren
	//Statusled fr den Lampenstatus 
	ledstatus = 0;	
	switch (controllerstatus)
	{
		case STAT_OFF: 		break; //Alle aus
		case STAT_ABBLEND: 	ledstatus |= (1<<LED_STAT_ON); break;	
		case STAT_FERN:		ledstatus |= (1<<LED_STAT_ON)|(1<<LED_STAT_BRIGHT); break;
	}		
		
	if(controllerstatus>STAT_OFF)
	{
		//Akkustand ermitteln
		akkuGetStatus();
	
		//Akkustatus => Aktionen
		ledstatus &= ~((1<<LED_PWR_ON)|(1<<LED_PWR_BRIGHT)|(1<<LED_PWR_BLINK));		
		switch (akkustatus)
		{
			case 0:	//blau hell blinkend
			{
				ledstatus = (1<<LED_STAT_ON)|(1<<LED_STAT_BRIGHT)|(1<<LED_STAT_BLINK);
				controllerstatus = STAT_ABBLEND;
				break;
			}
			case 1:	//rot und blau hell blinkend
			{
				ledstatus |= (1<<LED_STAT_ON)|(1<<LED_STAT_BRIGHT)|(1<<LED_STAT_BLINK);
				ledstatus |= (1<<LED_PWR_ON)|(1<<LED_PWR_BRIGHT)|(1<<LED_PWR_BLINK); 
				break;
			}
			case 2:	//rot hell blinkend
			{
				ledstatus |= (1<<LED_PWR_ON)|(1<<LED_PWR_BRIGHT)|(1<<LED_PWR_BLINK);
				break;
			}
			case 3:	//rot hell 
			{
				ledstatus |= (1<<LED_PWR_ON)|(1<<LED_PWR_BRIGHT);
				break;
			} 
			case 4:	//rot dunkel
			{
				ledstatus |= (1<<LED_PWR_ON); 	
				break;
			}
			case 5:	break; //PWR-Led aus
		}
	}
	
	ledRefresh();

	//FET Ausgnge setzen
	fetSetOutput();		
}

/***************************************************************************
 * 
 * Initialization
 * 
 ***************************************************************************/
void init(void)
{
	timerInitialize();	//Timerroutinen initalisieren
	buttonInitialize();	//Taster initialisieren
	ledInitialize();

	fetInitialize();	//PWM-Ausgnge initialisieren
	akkuInitialize();	//A/D-Wandler initiieren
	controllerstatus = 0;
	ledstatus = 0;
}

/***************************************************************************
 * 
 * Programmeinstiegsroutine
 * 
 ***************************************************************************/
int main(void)
{
 	//Alles initialisieren
	init();

	//Selbstest durchfhren
	doSelfTest();

	////Write Val to Eeprom
	//uint8_t temp;
	//temp = adcReadValue();
	//eeprom_write_byte(&eeFooByte, temp);	
	
	//ActionFlag auf 1, damit beim 1. Durchlauf alles einmal ausgefhrt wird
	doactionflag = 1;

	//Interrupts aktivieren
  	sei();	
	
	controllerstatus = 0;
	ledstatus = 0;

	//Programmschleife, Warten bis ActionFlag durch Timer auf 1 gesetzt wird
	while (1)
		if (doactionflag)
		{
			//Alle Aktionen ausfhren
			doAction();

			//ActionFlag lschen
			doactionflag = 0;
		}
	return 0;
}
